@@ -1,11 +1,13 @@
-import operator
-from collections import deque, OrderedDict
+import math
 
 import attr
 import outcome
 
-from . import _core
+import trio
+
 from ._util import aiter_compat
+from ._core import enable_ki_protection, ParkingLot
+from ._deprecate import deprecated
 
 __all__ = [
     "Event",
@@ -14,7 +16,6 @@ __all__ = [
     "Lock",
     "StrictFIFOLock",
     "Condition",
-    "Queue",
 ]
 
 
@@ -36,9 +37,16 @@ class Event:
     primitive that doesn't have this protection, consider :class:`Condition`
     or :class:`trio.hazmat.ParkingLot`.
 
+    .. note:: Unlike `threading.Event`, `trio.Event` has no
+       `~threading.Event.clear` method. In Trio, once an `Event` has happened,
+       it cannot un-happen. If you need to represent a series of events,
+       consider creating a new `Event` object for each one (they're cheap!),
+       or other synchronization methods like :ref:`channels <channels>` or
+       `trio.hazmat.ParkingLot`.
+
     """
 
-    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
+    _lot = attr.ib(factory=ParkingLot, init=False)
     _flag = attr.ib(default=False, init=False)
 
     def is_set(self):
@@ -47,7 +55,7 @@ class Event:
         """
         return self._flag
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def set(self):
         """Set the internal flag value to True, and wake any waiting tasks.
 
@@ -55,10 +63,12 @@ class Event:
         self._flag = True
         self._lot.unpark_all()
 
+    @deprecated(
+        "0.12.0",
+        issue=637,
+        instead="multiple Event objects or other synchronization primitives"
+    )
     def clear(self):
-        """Set the internal flag value to False.
-
-        """
         self._flag = False
 
     async def wait(self):
@@ -68,7 +78,7 @@ class Event:
 
         """
         if self._flag:
-            await _core.checkpoint()
+            await trio.hazmat.checkpoint()
         else:
             await self._lot.park()
 
@@ -85,14 +95,14 @@ class Event:
 
 
 def async_cm(cls):
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def __aenter__(self):
         await self.acquire()
 
     __aenter__.__qualname__ = cls.__qualname__ + ".__aenter__"
     cls.__aenter__ = __aenter__
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def __aexit__(self, *args):
         self.release()
 
@@ -143,9 +153,9 @@ class CapacityLimiter:
     fixed number of seats, and if they're all taken then you have to wait for
     someone to get up before you can sit down.
 
-    By default, :func:`run_sync_in_worker_thread` uses a
+    By default, :func:`trio.to_thread.run_sync` uses a
     :class:`CapacityLimiter` to limit the number of threads running at once;
-    see :func:`current_default_worker_thread_limiter` for details.
+    see `trio.to_thread.current_default_thread_limiter` for details.
 
     If you're familiar with semaphores, then you can think of this as a
     restricted semaphore that's specialized for one common use case, with
@@ -165,7 +175,7 @@ class CapacityLimiter:
     """
 
     def __init__(self, total_tokens):
-        self._lot = _core.ParkingLot()
+        self._lot = ParkingLot()
         self._borrowers = set()
         # Maps tasks attempting to acquire -> borrower, to handle on-behalf-of
         self._pending_borrowers = {}
@@ -198,8 +208,10 @@ class CapacityLimiter:
 
     @total_tokens.setter
     def total_tokens(self, new_total_tokens):
-        if not isinstance(new_total_tokens, int):
-            raise TypeError("total_tokens must be an int")
+        if not isinstance(
+            new_total_tokens, int
+        ) and new_total_tokens != math.inf:
+            raise TypeError("total_tokens must be an int or math.inf")
         if new_total_tokens < 1:
             raise ValueError("total_tokens must be >= 1")
         self._total_tokens = new_total_tokens
@@ -224,7 +236,7 @@ class CapacityLimiter:
         """
         return self.total_tokens - self.borrowed_tokens
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Borrow a token from the sack, without blocking.
 
@@ -234,9 +246,9 @@ class CapacityLimiter:
               tokens.
 
         """
-        self.acquire_on_behalf_of_nowait(_core.current_task())
+        self.acquire_on_behalf_of_nowait(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_on_behalf_of_nowait(self, borrower):
         """Borrow a token from the sack on behalf of ``borrower``, without
         blocking.
@@ -244,7 +256,7 @@ class CapacityLimiter:
         Args:
           borrower: A :class:`trio.hazmat.Task` or arbitrary opaque object
              used to record who is borrowing this token. This is used by
-             :func:`run_sync_in_worker_thread` to allow threads to "hold
+             :func:`trio.to_thread.run_sync` to allow threads to "hold
              tokens", with the intention in the future of using it to `allow
              deadlock detection and other useful things
              <https://github.com/python-trio/trio/issues/182>`__
@@ -263,9 +275,9 @@ class CapacityLimiter:
         if len(self._borrowers) < self._total_tokens and not self._lot:
             self._borrowers.add(borrower)
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Borrow a token from the sack, blocking if necessary.
 
@@ -274,9 +286,9 @@ class CapacityLimiter:
               tokens.
 
         """
-        await self.acquire_on_behalf_of(_core.current_task())
+        await self.acquire_on_behalf_of(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire_on_behalf_of(self, borrower):
         """Borrow a token from the sack on behalf of ``borrower``, blocking if
         necessary.
@@ -291,24 +303,21 @@ class CapacityLimiter:
              tokens.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_on_behalf_of_nowait(borrower)
-        except _core.WouldBlock:
-            task = _core.current_task()
+        except trio.WouldBlock:
+            task = trio.hazmat.current_task()
             self._pending_borrowers[task] = borrower
             try:
                 await self._lot.park()
-            except _core.Cancelled:
+            except trio.Cancelled:
                 self._pending_borrowers.pop(task)
                 raise
-        except:
-            await _core.cancel_shielded_checkpoint()
-            raise
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Put a token back into the sack.
 
@@ -317,9 +326,9 @@ class CapacityLimiter:
               sack's tokens.
 
         """
-        self.release_on_behalf_of(_core.current_task())
+        self.release_on_behalf_of(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release_on_behalf_of(self, borrower):
         """Put a token back into the sack on behalf of ``borrower``.
 
@@ -405,7 +414,7 @@ class Semaphore:
         # Invariants:
         # bool(self._lot) implies self._value == 0
         # (or equivalently: self._value > 0 implies not self._lot)
-        self._lot = _core.ParkingLot()
+        self._lot = trio.hazmat.ParkingLot()
         self._value = initial_value
         self._max_value = max_value
 
@@ -434,7 +443,7 @@ class Semaphore:
         """
         return self._max_value
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Attempt to decrement the semaphore value, without blocking.
 
@@ -446,23 +455,23 @@ class Semaphore:
             assert not self._lot
             self._value -= 1
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Decrement the semaphore value, blocking if necessary to avoid
         letting it drop below zero.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_nowait()
-        except _core.WouldBlock:
+        except trio.WouldBlock:
             await self._lot.park()
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Increment the semaphore value, possibly waking a task blocked in
         :meth:`acquire`.
@@ -514,7 +523,7 @@ class Lock:
 
     """
 
-    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
+    _lot = attr.ib(factory=ParkingLot, init=False)
     _owner = attr.ib(default=None, init=False)
 
     def __repr__(self):
@@ -539,7 +548,7 @@ class Lock:
         """
         return self._owner is not None
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Attempt to acquire the lock, without blocking.
 
@@ -548,32 +557,32 @@ class Lock:
 
         """
 
-        task = _core.current_task()
+        task = trio.hazmat.current_task()
         if self._owner is task:
             raise RuntimeError("attempt to re-acquire an already held Lock")
         elif self._owner is None and not self._lot:
             # No-one owns it
             self._owner = task
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Acquire the lock, blocking if necessary.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_nowait()
-        except _core.WouldBlock:
+        except trio.WouldBlock:
             # NOTE: it's important that the contended acquire path is just
             # "_lot.park()", because that's how Condition.wait() acquires the
             # lock as well.
             await self._lot.park()
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Release the lock.
 
@@ -581,7 +590,7 @@ class Lock:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        task = _core.current_task()
+        task = trio.hazmat.current_task()
         if task is not self._owner:
             raise RuntimeError("can't release a Lock you don't own")
         if self._lot:
@@ -613,7 +622,7 @@ class StrictFIFOLock(Lock):
     lock in strict first-come-first-served order.
 
     An example of when this is useful is if you're implementing something like
-    :class:`trio.ssl.SSLStream` or an HTTP/2 server using `h2
+    :class:`trio.SSLStream` or an HTTP/2 server using `h2
     <https://hyper-h2.readthedocs.io/>`__, where you have multiple concurrent
     tasks that are interacting with a shared state machine, and at
     unpredictable moments the state machine requests that a chunk of data be
@@ -661,7 +670,7 @@ class StrictFIFOLock(Lock):
     the same order that the state machine generated it.
 
     Currently, :class:`StrictFIFOLock` is simply an alias for :class:`Lock`,
-    but (a) this may not always be true in the future, especially if trio ever
+    but (a) this may not always be true in the future, especially if Trio ever
     implements `more sophisticated scheduling policies
     <https://github.com/python-trio/trio/issues/32>`__, and (b) the above code
     is relying on a pretty subtle property of its lock. Using a
@@ -699,7 +708,7 @@ class Condition:
         if not type(lock) is Lock:
             raise TypeError("lock must be a trio.Lock")
         self._lock = lock
-        self._lot = _core.ParkingLot()
+        self._lot = trio.hazmat.ParkingLot()
 
     def locked(self):
         """Check whether the underlying lock is currently held.
@@ -731,7 +740,7 @@ class Condition:
         """
         self._lock.release()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def wait(self):
         """Wait for another thread to call :meth:`notify` or
         :meth:`notify_all`.
@@ -755,7 +764,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to wait")
         self.release()
         # NOTE: we go to sleep on self._lot, but we'll wake up on
@@ -763,7 +772,7 @@ class Condition:
         try:
             await self._lot.park()
         except:
-            with _core.open_cancel_scope(shield=True):
+            with trio.CancelScope(shield=True):
                 await self.acquire()
             raise
 
@@ -777,7 +786,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to notify")
         self._lot.repark(self._lock._lot, count=n)
 
@@ -788,7 +797,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to notify")
         self._lot.repark_all(self._lock._lot)
 
@@ -807,210 +816,3 @@ class Condition:
             tasks_waiting=len(self._lot),
             lock_statistics=self._lock.statistics(),
         )
-
-
-@attr.s(frozen=True)
-class _QueueStats:
-    qsize = attr.ib()
-    capacity = attr.ib()
-    tasks_waiting_put = attr.ib()
-    tasks_waiting_get = attr.ib()
-
-
-# Like queue.Queue, with the notable difference that the capacity argument is
-# mandatory.
-class Queue:
-    """A bounded queue suitable for inter-task communication.
-
-    This class is generally modelled after :class:`queue.Queue`, but with the
-    major difference that it is always bounded.
-
-    A :class:`Queue` object can be used as an asynchronous iterator that
-    dequeues objects one at a time. That is, these two loops are equivalent::
-
-       async for obj in queue:
-           ...
-
-       while True:
-           obj = await queue.get()
-           ...
-
-    Args:
-      capacity (int): The maximum number of items allowed in the queue before
-          :meth:`put` blocks. Choosing a sensible value here is important to
-          ensure that backpressure is communicated promptly and avoid
-          unnecessary latency. If in doubt, use 0.
-
-    """
-
-    def __init__(self, capacity):
-        if not isinstance(capacity, int):
-            raise TypeError("capacity must be an integer")
-        if capacity < 0:
-            raise ValueError("capacity must be >= 0")
-        self.capacity = operator.index(capacity)
-        # {task: None} ordered set
-        self._get_wait = OrderedDict()
-        # {task: queued value}
-        self._put_wait = OrderedDict()
-        # invariants:
-        # if len(self._data) < self.capacity, then self._put_wait is empty
-        # if len(self._data) > 0, then self._get_wait is empty
-        self._data = deque()
-
-    def __repr__(self):
-        return (
-            "<Queue({}) at {:#x} holding {} items>".format(
-                self.capacity, id(self), len(self._data)
-            )
-        )
-
-    def qsize(self):
-        """Returns the number of items currently in the queue.
-
-        There is some subtlety to interpreting this method's return value: see
-        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.
-
-        """
-        return len(self._data)
-
-    def full(self):
-        """Returns True if the queue is at capacity, False otherwise.
-
-        There is some subtlety to interpreting this method's return value: see
-        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.
-
-        """
-        return len(self._data) == self.capacity
-
-    def empty(self):
-        """Returns True if the queue is empty, False otherwise.
-
-        There is some subtlety to interpreting this method's return value: see
-        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.
-
-        """
-        return not self._data
-
-    @_core.enable_ki_protection
-    def put_nowait(self, obj):
-        """Attempt to put an object into the queue, without blocking.
-
-        Args:
-          obj (object): The object to enqueue.
-
-        Raises:
-          WouldBlock: if the queue is full.
-
-        """
-        if self._get_wait:
-            assert not self._data
-            task, _ = self._get_wait.popitem(last=False)
-            _core.reschedule(task, outcome.Value(obj))
-        elif len(self._data) < self.capacity:
-            self._data.append(obj)
-        else:
-            raise _core.WouldBlock()
-
-    @_core.enable_ki_protection
-    async def put(self, obj):
-        """Put an object into the queue, blocking if necessary.
-
-        Args:
-          obj (object): The object to enqueue.
-
-        """
-        await _core.checkpoint_if_cancelled()
-        try:
-            self.put_nowait(obj)
-        except _core.WouldBlock:
-            pass
-        else:
-            await _core.cancel_shielded_checkpoint()
-            return
-
-        task = _core.current_task()
-        self._put_wait[task] = obj
-
-        def abort_fn(_):
-            del self._put_wait[task]
-            return _core.Abort.SUCCEEDED
-
-        await _core.wait_task_rescheduled(abort_fn)
-
-    @_core.enable_ki_protection
-    def get_nowait(self):
-        """Attempt to get an object from the queue, without blocking.
-
-        Returns:
-          object: The dequeued object.
-
-        Raises:
-          WouldBlock: if the queue is empty.
-
-        """
-        if self._put_wait:
-            task, value = self._put_wait.popitem(last=False)
-            # No need to check max_size, b/c we'll pop an item off again right
-            # below.
-            self._data.append(value)
-            _core.reschedule(task)
-        if self._data:
-            value = self._data.popleft()
-            return value
-        raise _core.WouldBlock()
-
-    @_core.enable_ki_protection
-    async def get(self):
-        """Get an object from the queue, blocking if necessary.
-
-        Returns:
-          object: The dequeued object.
-
-        """
-        await _core.checkpoint_if_cancelled()
-        try:
-            value = self.get_nowait()
-        except _core.WouldBlock:
-            pass
-        else:
-            await _core.cancel_shielded_checkpoint()
-            return value
-
-        # Queue doesn't have anything, we must wait.
-        task = _core.current_task()
-
-        def abort_fn(_):
-            self._get_wait.pop(task)
-            return _core.Abort.SUCCEEDED
-
-        self._get_wait[task] = None
-        value = await _core.wait_task_rescheduled(abort_fn)
-        return value
-
-    @aiter_compat
-    def __aiter__(self):
-        return self
-
-    async def __anext__(self):
-        return await self.get()
-
-    def statistics(self):
-        """Returns an object containing debugging information.
-
-        Currently the following fields are defined:
-
-        * ``qsize``: The number of items currently in the queue.
-        * ``capacity``: The maximum number of items the queue can hold.
-        * ``tasks_waiting_put``: The number of tasks blocked on this queue's
-          :meth:`put` method.
-        * ``tasks_waiting_get``: The number of tasks blocked on this queue's
-          :meth:`get` method.
-
-        """
-        return _QueueStats(
-            qsize=len(self._data),
-            capacity=self.capacity,
-            tasks_waiting_put=len(self._put_wait),
-            tasks_waiting_get=len(self._get_wait),
-        )
